<div style="position: relative;overflow: hidden;" class="{{ css_class }}">
    Upload file
    <form style="position: absolute;right: 0;top: 0;cursor: hand;opacity: 0;"
          id="{{ id }}-form">
        <input style="cursor: pointer" id="{{ id }}" type="file" name="ajax_file">
    </form>
</div>
<script>
    $('#{{ id }}').change(function () {
        ajax_helpers.send_form('{{ id }}-form')
    })
</script>